#pragma once

#include <inttypes.h>


typedef uint16_t RingBuffer16Size;
typedef uint16_t RingBuffer16Data;

//! This struct represents a ring buffer.
struct SRingBuffer16
{
	//! The size of the buffer
	volatile RingBuffer16Size		size;
	//! An array that contains the data of the buffer.
	RingBuffer16Data		* volatile data;
	//! The head of the buffer.
	volatile RingBuffer16Size	head;
	//! The tail of the buffer.
	volatile RingBuffer16Size	tail;
};



/**
 * \brief					Add a word to the end of a buffer.
 * \param	word			The word to add.
 * \param	ring_buffer		A pointer to the buffer that the word is added to.
 */
void RingBuffer16_Push(RingBuffer16Data word, struct SRingBuffer16 * ring_buffer);

/**
 * \brief					Read a word from the beginning of a buffer.
 * \param	ring_buffer		A pointer to the buffer that the word is read from.
 * \return					The read word.
 */
RingBuffer16Data RingBuffer16_Pop( struct SRingBuffer16 * ring_buffer);

/**
 * \brief					Get the number of words that a buffer contains.
 * \param	ring_buffer		A pointer to the buffer of that the size should be read out.
 */
RingBuffer16Size	RingBuffer16_Size( struct SRingBuffer16 * ring_buffer);

void RingBuffer16_Clear( struct SRingBuffer16 * ring_buffer);

/**
 * \brief					Does the first-time initialisation of a ring buffer.
 * \param	ring_buffer		A pointer to the buffer that should be initialised.
 */
void RingBuffer16_Init( struct SRingBuffer16 * ring_buffer, RingBuffer16Data * volatile data, RingBuffer16Size size);
