#pragma once

#include <inttypes.h>


typedef uint8_t RingBuffer8Size;
typedef uint8_t RingBuffer8Data;

//! This struct represents a ring buffer.
struct SRingBuffer8
{
	//! The size of the buffer
	volatile RingBuffer8Size		size;
	//! An array that contains the data of the buffer.
	RingBuffer8Data		* volatile data;
	//! A pointer to the byte next read.
	volatile RingBuffer8Size	head;
	//! A pointer to the byte next written.
	volatile RingBuffer8Size	tail;
};



/**
 * \brief					Add a byte to the end of a buffer.
 * \param	byte			The byte to add.
 * \param	ring_buffer		A pointer to the buffer that the byte is added to.
 */
void RingBuffer8_Push(RingBuffer8Data byte, struct SRingBuffer8 * ring_buffer);

/**
 * \brief					Read a byte from the beginning of a buffer.
 * \param	ring_buffer		A pointer to the buffer that the byte is read from.
 * \return					The read byte.
 */
RingBuffer8Data RingBuffer8_Pop( struct SRingBuffer8 * ring_buffer);

/**
 * \brief					Get the number of bytes that a buffer contains.
 * \param	ring_buffer		A pointer to the buffer of that the size should be read out.
 */
RingBuffer8Size	RingBuffer8_Size( struct SRingBuffer8 * ring_buffer);

void RingBuffer8_Clear( struct SRingBuffer8 * ring_buffer);

/**
 * \brief					Does the first-time initialisation of a ring buffer.
 * \param	ring_buffer		A pointer to the buffer that should be initialised.
 */
void RingBuffer8_Init( struct SRingBuffer8 * ring_buffer, RingBuffer8Data * volatile data, RingBuffer8Size size);
